#ifndef EXP_H
#define EXP_H
//  DtL - Derive to LaTeX by Gary Bishop 29 June 1999

// $Id: exp.h,v 1.1 1999-07-01 11:03:08-04 gb Exp gb $

#include "Str.h"

// this class represents the syntax tree of the recognized expression. I use this to
// generate the LaTeX expression.

class exp {
 public:
  enum exp_type { evar, // variable
		  econst, // numbers and such
		  ecall, // a function call
		  eunary, // a unary operator like - in -a.
		  ebinary, // a binary operator like + in a+b.
		  etimes, // multiply
		  edivide, // division
		  elist, // a list of expressions as in foo(a, b, c)
		  evector, // a vector of expressions as in [a, b, c]
		  eintegral, // an integral
		  esumprod, // summation or product
		  elimit, // a limit
		  escript, // a subscript as in A SUB i or superscript as in A^i
		  ebracket, // a operation like ABS that is done with brackets like |A|
  } etype;
  int level; // this describes the precedence of the operator seen from above in the tree

  exp(exp_type e, int l) { etype = e; level = l; }

  virtual Str latex() = 0; // we redifine this in derived types
  Str paren(int level);    // parenthesize the sub-expression if necessary
};

class exp_var : public exp {
 public:
  Str name;
  exp_var(Str nm) : exp(evar, 70) { name = nm; }
  Str latex();
};

class exp_const : public exp {
 public:
  Str val;
  exp_const(Str vl) : exp(econst, 70) { val = vl; }
  Str latex();
};

class exp_list : public exp {
 public:
  int ecnt;
  exp* head;
  exp_list* tail;
  exp_list(exp* h, exp_list* t=0) : exp(elist, 10)
    { head = h; tail = t; if(tail == 0) ecnt=1; else ecnt = tail->ecnt+1; }
  Str latex();
};

class exp_vector : public exp {
 public:
  exp_list* elems;
  exp_vector(exp_list* el) : exp(evector, 100)
    { elems = el; }
  Str latex();
};

class exp_call : public exp {
 public:
  Str fun;
  int arg_level;
  exp* args;
  exp_call(Str f, exp* a, int al) : exp(ecall, 60) { fun=f; args=a; arg_level = al; }
  Str latex();
};

class exp_unary : public exp {
 public:
  char* op;
  exp* opr;
  exp_unary(char* o, exp* or) : exp(eunary, 55) { op=o; opr=or; }
  Str latex();
};

class exp_binary : public exp {
 public:
  Str op;
  exp* left;
  exp* right;
  exp_binary(Str o, int lv, exp*l, exp*r) : exp(ebinary, lv) { op=o; left=l; right=r; }
  Str latex();
};

class exp_times : public exp {
 public:
  exp* left;
  exp* right;
  exp_times(exp* l, exp* r) : exp(etimes, 30) { left=l; right=r; }
  Str latex();
};

class exp_divide : public exp {
 public:
  exp* left;
  exp* right;
  exp_divide(exp* l, exp* r) : exp(edivide, 40) { left=l; right=r; }
  Str latex();
};

class exp_integral : public exp {
 public:
  exp_list *args;
  exp_integral(exp_list* a) : exp(eintegral, 0) { args=a; }
  Str latex();
};

class exp_sumprod : public exp {
 public:
  Str op;
  exp_list *args;
  exp_sumprod(const char* o, exp_list* a) : exp(esumprod, 0) { op = o; args=a; }
  Str latex();
};

class exp_limit : public exp {
 public:
  exp_list *args;
  exp_limit(exp_list* a) : exp(elimit, 0) { args=a; }
  Str latex();
};

class exp_script : public exp {
 public:
  Str op;
  exp* vec;
  exp* script;
  exp_script(const char* o, exp* v, exp* s) : exp(escript, 100) { op = o; vec=v; script=s; }
  Str latex();
};

class exp_bracket : public exp {
 public:
  Str lbracket;
  Str rbracket;
  exp* operand;
  exp_bracket(const char* l, const char* r, exp* o) : exp(ebracket, 100)
    { lbracket=l; rbracket=r; operand=o; }
  Str latex();
};

#endif
